/*
 *  wmpop3.c
 *  multi pop3 mail checker. 
 *  written by Louis-Benoit JOURDAIN (lb@jourdain.org)
 *  based on the original work by Scott Holden (scotth@thezone.net)
 *
*/


#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>

#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>

#include <X11/xpm.h>
#include <X11/Xlib.h>
#include <X11/extensions/shape.h>

#include "../wmgeneral/misc.h"
#include "../wmgeneral/wmgeneral.h"
#include "Pop3Client.h"

#include "wmpop3.xpm"

char wminet_mask_bits[64*64];
int  wminet_mask_width = 64;
int  wminet_mask_height = 64;

char *ProgName;

char mailclient[32]   = "pine";
char password[32];
char username[32];
char popserver[128];
int  serverport       = 110;
int  mailCheckDelay   = 10;  /* default */
int  autoChecking     = YES; /* default */
int  newMessagesOnly  = YES; /* default */
int  displaydelay = 1;
char config_file[256] = "not-defined";

Pop3	conf[6];
int	nb_conf;	/* number of configured pop servers */
int	summess;	/* total number of messages */
int	color;


void usage(void);
void printversion(void);
void wmCheckMail_routine(int, char **);
int  readConfigFile( char *filename );

void BlitString(char *name, int x, int y);
void BlitNum(int num, int x, int y, int todelete);

int	pop3DeleteMail(int num, Pop3 pc);
int	pop3VerifStats(Pop3 pc);

/********************************
 *           Main Program
 ********************************/
int main(int argc, char *argv[]) {

	int		i;

	ProgName = argv[0];
	if (strlen(ProgName) >= 5)
		ProgName += (strlen(ProgName) - 5);
	
	for (i=1; i<argc; i++) {
		char *arg = argv[i];

		if (*arg=='-') {
			switch (arg[1]) {
			case 'd' :
				if (strcmp(arg+1, "display")) {
					usage();
					exit(1);
				}
				break;
			case 'g' :
				if (strcmp(arg+1, "geometry")) {
					usage();
					exit(1);
				}
				break;
			case 'v' :
				printversion();
				exit(0);
				break;
            case 'c' :
                if (argc > (i+1))
                {
                    strcpy(config_file, argv[i+1]);
                    i++;
                }
                break;
            default:
				usage();
				exit(0);
				break;
			}
		}
	}

	wmCheckMail_routine(argc, argv);

	return 0;
}

void		build_line(char *buf, int num, int index, Pop3 pc) 
{
  int		len;
  int		pos;
  char		tmp[256];
  
  memset(tmp, 0, sizeof(char) * 256);
  memcpy(tmp, pc->alias, 3);
  tmp[3] = ':';
  pos = 4;
  for (len = 0; len < MAIL_BUFLEN && pc->mails[num].from[len]; len++);
  memcpy(tmp + pos, pc->mails[num].from, len);
  tmp[pos + len] = '/';
  pos += len + 1;
  for (len = 0; len < MAIL_BUFLEN && pc->mails[num].subject[len]; len++);
  memcpy(tmp + pos, pc->mails[num].subject, len);
  len += pos;
  tmp[len++] = ' ';
  tmp[len++] = '*';
  tmp[len++] = ' ';
  tmp[len] = '\0';
  pos = index % len;
  if (len - pos < NB_DISP) { /* We are at the end of the string */
    memcpy(buf, tmp + pos, len - pos);
    memcpy(buf + len - pos, tmp, NB_DISP - len + pos);
  }
  else {
    fflush(stdout);
    memcpy(buf, tmp + pos, NB_DISP);
  }
}

/* display the mail number in the left column */
void		display_index(int mail_index, int line, Pop3 pc)
{
  BlitNum(mail_index + 1, LEFT_MARGIN, (line * 6) + TOP_MARGIN,
	  pc->mails[mail_index].todelete);
}

/* return < 0 if error
   0 if no pb
   else nb of mails which couldn't be deleted */
int	DeleteMail(Pop3 pc)
{
  int	etat = 0;
  int	old_sizeOfAllMessages;
  int	i;
  
  old_sizeOfAllMessages = pc->sizeOfAllMessages;
  BlitString(pc->alias, 10, TOP_MARGIN);
  BlitString("connect", 10, 6*2 + TOP_MARGIN);
  RedrawWindow();
  etat = pop3MakeConnection(pc, pc->popserver, pc->serverport);
  if (-1 != etat) {
    BlitString("login", 10, (6*3) + TOP_MARGIN);
    RedrawWindow();
    etat = pop3Login(pc, pc->username, pc->password);
  }
  if (-1 != etat) {
    BlitString("chk cont", 10, (6*4) + TOP_MARGIN);
    RedrawWindow();
    etat = pop3VerifStats(pc);
  }
  if (-1 != etat) {
    if (etat != old_sizeOfAllMessages) {
      BlitString("mailbox", 10, TOP_MARGIN);
      BlitString("content", 10, 6 + TOP_MARGIN);
      BlitString("changed", 10, 6*2 + TOP_MARGIN);
      BlitString("between", 10, 6*3 + TOP_MARGIN);
      BlitString("updates", 10, 6*4 + TOP_MARGIN);
      BlitString("del can.", 10, 6*6 + TOP_MARGIN);
      RedrawWindow();
      sleep(displaydelay);
      for (i = 0, etat = 0; i < pc->numOfMessages && i < MAX_NB_MSG; i++) {
	etat += pc->mails[i].todelete;
      }
    }
    else {
      etat = 0;
      for (i = 0; i < pc->numOfMessages && i <MAX_NB_MSG; i++) {
	if (pc->mails[i].todelete) {
	  etat += pop3DeleteMail(i + 1, pc);
	}
      }
    }
  }
  pop3Quit(pc);
  return (etat);
}  


void		launch_mail_client(int iS)
{
  int		i;
  int		j;
  char		str[16];
  char		**args;
  
  /* clear the display */
  copyXPMArea(72, 4, 3, 42, LEFT_MARGIN, 4);
  copyXPMArea(72, 4, 50, 42, 9, 4 );
  if ((args = conf[iS]->mailclient)) {
    BlitString("starting", 10, TOP_MARGIN);
    for (i = 0; i < 5 && args[i]; i++) {
      for (j = 0; j < 8 && args[i][j]; j++);
      memcpy(str, args[i], j);
      str[j] = '\0';
      BlitString(str, 10, (i + 2)*6 + TOP_MARGIN);
    }
    if (0 == fork()) {
      if (execvp(args[0], args)) {
	perror("execvp");
	/* clear the first line */
	copyXPMArea(72, 4, 3, 42, LEFT_MARGIN, 4);
	copyXPMArea(72, 4, 50, 6, 9, 4 );
	BlitString("Error", 10, TOP_MARGIN);
      }
    }
    else {
      /* set the check delay to 30 seconds */
      conf[iS]->nextCheckTime = time(0) + 30;
    }
  }
  else {
    BlitString("  mail", 10, TOP_MARGIN);
    BlitString(" client", 10, 6 + TOP_MARGIN);
    BlitString("not  set", 10, 6*2 + TOP_MARGIN);
    BlitString(" check", 15, 6*5 + TOP_MARGIN);
    BlitString("wmpop3rc", 10, 6*6 + TOP_MARGIN);
  }
  RedrawWindow();
  sleep(displaydelay);
}

/*
 * wmCheckMail_routine : This function is used to set up the X display
 *           for wmpop3 and check for mail every n number of minutes.
 */

void		wmCheckMail_routine(int argc, char **argv){
  
  int		buttonStatus   = -1;
  int		iS;
  Pop3		pc;
  int		i, j;
  XEvent	Event;
  char		str[256];
  int		index = 0;
  int		index_vert = 0;
  int		oldnbmess = 0;
  int		nbsel;
  long		thistime;
  char		*linestodel[7];
  int		unreachable;
    
  if( !strcmp( config_file, "not-defined") )
    sprintf(config_file, "%s/.wmpop3rc", getenv("HOME"));
  
  if( readConfigFile(config_file) == -1){
    exit(0);
  }
  /* Set up timer for checking mail */
  createXBMfromXPM(wminet_mask_bits, wmpop3_xpm
		   , wminet_mask_width, wminet_mask_height);
  
  openXwindow(argc, argv, wmpop3_xpm, wminet_mask_bits
	      , wminet_mask_width, wminet_mask_height);
  
  AddMouseRegion(0, 19, 49, 30, 58); /* check button */
  AddMouseRegion(1, 46, 49, 50, 58 ); /* autocheck button   */
  AddMouseRegion(2, 53, 49, 57, 58 ); /* switch display button  */
  AddMouseRegion(3, 5, 49, 16, 58 ); /* delete button */ 
  AddMouseRegion(4, 33, 49, 44, 52); /* top arrow */
  AddMouseRegion(5, 33, 54, 44, 58); /* botton arrow */
  /* add the mouse regions for each line */
  for (i = 0; i < NB_LINE; i++) {
    AddMouseRegion(6 + i, 9, (i*6) + TOP_MARGIN, 
		   58, (i*6) + TOP_MARGIN + CHAR_HEIGHT);
  }
  
  
  /* Check if Autochecking is on or off */
  if(autoChecking == NO ){
    copyXPMArea(142, 38, 7, 11, 45, 48);
  }
  else {
    copyXPMArea(142, 49, 7, 11, 45, 48);
  }
  
  RedrawWindow();
  
  summess = 0;
  while (1) {
    RSET_COLOR;
    for (iS = 0; iS < nb_conf; iS++) {
      pc = conf[iS];
      if( (((time(0) > pc->nextCheckTime) || 
	    (pc->nextCheckTime == 0)) && ( autoChecking == YES)) 
	  || (pc->forcedCheck == YES)){ 
	/* clear the display */
	copyXPMArea(72, 4, 3, 42, LEFT_MARGIN, 4);
	copyXPMArea(72, 4, 50, 42, 9, 4 );
	BlitString(pc->alias, 10, TOP_MARGIN);
	BlitString("connect", 10, (6*2) + TOP_MARGIN);
	RedrawWindow();
	pc->status = 0;
	if(pop3MakeConnection(pc,pc->popserver,pc->serverport) == -1){
	  pc->status = 1;
	}
	if (!pc->status) {
	  BlitString("login", 10, (6*3) + TOP_MARGIN);
	  RedrawWindow();
	  if (pop3Login(pc, pc->username, pc->password) == -1 ) 
	    pc->status = 2;
	}
	if (!pc->status) {
	  BlitString("get mail", 10, (6*4) + TOP_MARGIN);
	  RedrawWindow();
	  if (pop3CheckMail(pc) == -1)
	    pc->status = 3;
	}
	pop3Quit(pc);
	pc->nextCheckTime = time(0) + (pc->mailCheckDelay * SEC_IN_MIN);
	pc->forcedCheck = NO;
	index = 0;
      }
    }
    waitpid(0, NULL, WNOHANG);
    /* reset the number of messages */
    unreachable = 1;
    for (summess = 0, iS = 0; iS < nb_conf; iS++) {
      pc = conf[iS];
      if (!pc->status) {
	unreachable = 0;
	summess += 
	  (pc->numOfMessages > MAX_NB_MSG) ? MAX_NB_MSG : pc->numOfMessages;
      }
    }
    /* set the offset from the beginning of the list */
    if (summess != oldnbmess) {
      oldnbmess = summess;
      if (summess < NB_LINE)
	index_vert = 0;
      else {
	index_vert = summess - NB_LINE;
      }
    }
    
    memset(str, 0, 128);
    /*      if( mailWaiting == -1 ){
	    we don't clear the display so that we know what was the error 
	    BlitString(":error:", 10, (6*6) + TOP_MARGIN);
	    } */
    
    /* clear the display */
    copyXPMArea(72, 4, 3, 42, LEFT_MARGIN, 4);
    copyXPMArea(72, 4, 50, 42, 9, 4 );
    if (newMessagesOnly == YES ){
      /* Show messages waiting */
      nbsel = 0;
      RSET_COLOR;
      for (color = 0, iS = 0; iS < nb_conf; iS++) {
	pc = conf[iS];
	switch(pc->status) {
	case 1:
	  sprintf(str, "%-3s:C*ER", pc->alias);
	  break;
	case 2:
	  sprintf(str, "%-3s:L*ER", pc->alias);
	  break;
	case 3:
	  sprintf(str, "%-3s:M*ER", pc->alias);
	  break;
	default:
	  sprintf(str, "%-3s: %3d", pc->alias, 
		  (pc->countunreadonly) ? pc->numOfUnreadMessages : 
		  pc->numOfMessages);
	  for (i = 0; i < summess && i < pc->numOfMessages; i++)
	    nbsel += pc->mails[i].todelete;
	  break;
	}
	BlitString(str, 10, (int) (6*iS) + TOP_MARGIN);
	SWAP_COLOR;
      }
      RSET_COLOR;
      sprintf(str, "sel.: %2d", nbsel);
      BlitString(str, 10, (int) (6*6) + TOP_MARGIN);	  
    }
    else {
      RSET_COLOR;
      if (0 == summess) {
	if (unreachable) {
	  BlitString(" error", 10, TOP_MARGIN);
	}
	else {
	  BlitString("No  Mesg", 10, TOP_MARGIN);
	}
	if (autoChecking == YES) {
	  BlitString("  next", 10, 6*2 + TOP_MARGIN);
	  BlitString(" update", 10, 6*3 + TOP_MARGIN);
	  j = SEC_IN_MIN * 1000; 
	  thistime = time(0);
	  for (i = 0, nbsel = 0; i < nb_conf; i++) {
	    if ((conf[i]->nextCheckTime - thistime) < j) {
	      j = conf[i]->nextCheckTime - thistime;
	      nbsel = i;
	    }
	  }
	  sprintf(str, " is:%s", conf[nbsel]->alias);
	  BlitString(str, 10, 6*4 + TOP_MARGIN);
	  BlitString("   in", 10, 6*5 + TOP_MARGIN);
	  sprintf(str, "%-5d s.", j);
	  BlitString(str, 10, 6*6 + TOP_MARGIN);
	}
      }
      else {
	RSET_COLOR;
	/* iS = index in the conf struct
	   i = index of the mail 
	   j = line nb on display */
	memset(linestodel, 0, sizeof(char *));
	for (iS = 0, j = 0; iS < nb_conf && j < NB_LINE + index_vert; iS++) {
	  pc = conf[iS];
	  for (i = 0; (j < NB_LINE + index_vert) && i < pc->numOfMessages &&
		 i < MAX_NB_MSG; j++, i++) {
	    if (j >= index_vert) {
	      build_line(str, i, index, pc);
	      BlitString(str, 10, ((j - index_vert) * 6) + TOP_MARGIN);
	      /* store the address of the delete flag, so that it will */
	      /* be easier to modify it afterwards */
	      linestodel[j - index_vert] = &(pc->mails[i].todelete);
	      display_index(i, (j - index_vert), pc);
	    }
	  }
	  SWAP_COLOR;
	}
      }
      index++;
    }
    
    RedrawWindow();
    
    RSET_COLOR;
    /* X Events */
    while (XPending(display)){
      XNextEvent(display, &Event);
      switch (Event.type) {
      case Expose:
	RedrawWindow();
	break;
      case DestroyNotify:
	XCloseDisplay(display);
	exit(0);
	break;
      case ButtonPress:
	i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
	buttonStatus = i;
	if (buttonStatus == i && buttonStatus >= 0){
	  switch (buttonStatus){
	  case 0 : /* check button pressed */
	    copyXPMArea(128,38 ,14 ,11 ,18 ,48 );
	    break;
	  case 1 : /* autocheck button pressed */
	    break;
	  case 2: /* switch display button pressed */
	    break;
	  case 3: /* delete button pressed */
	    copyXPMArea(127, 15, 14, 11, 4, 48);
	    break;
	  case 4: /* top arrow button pressed */
	    break;
	  case 5: /* bottom arrow button pressed */
	    break;
	  default:
	    break;
	  }
	  RedrawWindow();
	    }
	break;
      case ButtonRelease:
	i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
	
	if (buttonStatus == i && buttonStatus >= 0){
	  switch (buttonStatus){
	  case 0 :   /* check button */
	    copyXPMArea(128,27 ,14 ,11 ,18 ,48 );	
	    for (iS = 0; iS < nb_conf; iS++)
	      conf[iS]->forcedCheck = YES;
	    break;
	  case 1 :   /* autocheck Button */
	    if (autoChecking == YES){
	      autoChecking = NO;
	      copyXPMArea(142, 38, 7, 11, 45, 48);		  
	    }
	    else {
	      autoChecking = YES;
	      for (iS = 0; iS < nb_conf; iS++)
		conf[iS]->nextCheckTime = time(0) + 
		  (conf[iS]->mailCheckDelay * SEC_IN_MIN);
	      copyXPMArea(142, 49, 7, 11, 45, 48);
	    }
	    break;
	  case 2:  /* switch display Button */
	    index = 0;
	    /* change view on # of messages */
	    if( newMessagesOnly == YES ) {
	      newMessagesOnly = NO;
	      copyXPMArea(149,38 , 7 , 11, 52, 48);
	    } 
	    else {
	      newMessagesOnly = YES;
	      copyXPMArea(149,49 , 7 , 11, 52, 48);
	    }
	    break;
	  case 3: /* delete button */
	    copyXPMArea(143, 15, 14, 11, 4, 48);
	    j = 0;
	    RSET_COLOR;
	    for (iS = 0; iS < nb_conf; iS++) {
	      pc = conf[iS];
	      for (i = 0, nbsel = 0;
		   i < pc->numOfMessages && i < MAX_NB_MSG; i++) 
		nbsel += pc->mails[i].todelete;
	      if (nbsel) {
		/* clear the display */
		copyXPMArea(72, 4, 3, 42, LEFT_MARGIN, 4);
		copyXPMArea(72, 4, 50, 42, 9, 4 );
		nbsel = DeleteMail(pc);
		if (nbsel < 0) {
		  sprintf(pc->delstatus, "%-3s: Err", pc->alias);
		}
		else if (nbsel > 0) {
		  sprintf(pc->delstatus, "%-3s/D:%2d", pc->alias, nbsel);
		}
		else 
		  sprintf(pc->delstatus, "%-3s:  ok", pc->alias);
		pc->forcedCheck = YES;
	      }
	      else {
		sprintf(pc->delstatus, "%-3s:none", pc->alias);
	      }
	      SWAP_COLOR;
	    }
	    /* clear the display */
	    copyXPMArea(72, 4, 3, 42, LEFT_MARGIN, 4);
	    copyXPMArea(72, 4, 50, 42, 9, 4 );
	    RSET_COLOR;
	    for (iS = 0; iS < nb_conf; iS++) {
	      BlitString(conf[iS]->delstatus, 10, 6*iS + TOP_MARGIN);
	      SWAP_COLOR;
	    }
	    RedrawWindow();
	    sleep(displaydelay);
	    break;
	  case 4: /* top arrow button pressed */
	    index_vert--;
	    if (0 > index_vert)
	      index_vert = 0;
	    break;
	  case 5: /* bottom arrow button pressed */
	    if (summess > NB_LINE) {
	      index_vert++;
	      if (index_vert + NB_LINE > summess)
		index_vert = summess - NB_LINE;
	    }
	    break;
	  default:
	    /* don't do anything if we are not in the correct view mode */
	    if (newMessagesOnly == NO) {
	      if ((5 < buttonStatus) && (buttonStatus <= 5 + NB_LINE)) {
		if ((buttonStatus - 6 + index_vert) < summess) {
		  *(linestodel[buttonStatus - 6]) = 
		    1 - *(linestodel[buttonStatus - 6]);
		}
	      }
	    }
	    else {
	      if ((5 < buttonStatus) && (buttonStatus <= 5 + nb_conf)) {
		launch_mail_client(buttonStatus - 6);
	      }
	    }
	    break;
	  }
	}
	RedrawWindow();
	buttonStatus = -1;
	break;
      }
    }
    usleep(100000L);
  }
}


/*
 * usage : Prints proper command parameters of wmpop3.
 */
void usage(void)
{
  fprintf(stdout, "\nWMPop3LB - Louis-Benoit JOURDAIN (lb@jourdain.org)\n");
  fprintf(stdout, "based on the work by Scott Holden <scotth@thezone.net>\n\n");
  fprintf(stdout, "usage:\n");
  fprintf(stdout, "    -display <display name>\n");
  fprintf(stdout, "    -geometry +XPOS+YPOS      initial window position\n");
  fprintf(stdout, "    -c <filename>             use specified config file\n");
  fprintf(stdout, "    -h                        this help screen\n");
  fprintf(stdout, "    -v                        print the version number\n");
  fprintf(stdout, "\n");
}

/*
 * printversion : This function is used to print the version info
 *                to standard output.
 */
void printversion(void)
{
  fprintf(stdout, "wmpop3 v%s\n", WMPOP3_VERSION);
}

// Blits a string at given co-ordinates
void BlitString(char *name, int x, int y)
{
  int		i;
  int		c;
  int		k;
  int		row;
  
  for (i = 0, k = x; name[i]; i++, k += CHAR_WIDTH + 1) {
    c = toupper(name[i]); 
    if (c >= 'A' && c <= 'Z') {   /* its a letter */
      c -= 'A';
      row = LETTERS;
    }
    else if (c >= '0' && c <= '9') {   /* its a number */
      c -= '0';
      row = NUMBERS;
    }
    else switch(c) {
    case '-':
      c = 26;
      row = LETTERS;
      break;
    case '*':
      c = 27;
      row = LETTERS;
      break;
    case ':': 
      c = 10;
      row = NUMBERS;
      break;
    case '/':
      c = 11;
      row = NUMBERS;
      break;
    case '@':
      c = 12;
      row = NUMBERS;
      break;
    case ' ':
      c = 13;
      row = NUMBERS;
      break;
    case '.':
      c = 28;
      row = LETTERS;
      break;
    default:
      c = 14;
       row = NUMBERS;
       break;
    }
    copyXPMArea(c * CHAR_WIDTH, CH_COLOR(row), CHAR_WIDTH + 1, CHAR_HEIGHT, 
		k, y);
  }
}


/* Blits number to give coordinates...*/

void BlitNum(int num, int x, int y, int todelete)
{
  if (MAX_NB_MSG < num)
    return ;
  if (todelete)
    num += 19;
  copyXPMArea(((num - 1) * (SN_CHAR_W + 1)) + 1, CH_COLOR(SMALL_NUM), 
	      SN_CHAR_W, CHAR_HEIGHT + 1, x, y);
}
    
int	parsestring(char *buf, char *data, int max) 
{
  char	*deb;
  char	*end;

  /* trim the leading spaces */
  memset(data, 0, max);
  for (deb = buf; *deb && isspace(*deb); deb++);
  if (!*deb)
    return (-1);
  if ('"' == *deb) {
    for (end = ++deb; *end && ('"' != *end); end++);
    if (!*end)
      return (-1);
    memcpy(data, deb, ((end - deb) > max) ? max : end - deb);
  }
  else {
    for (end = deb; *end && !isspace(*end); end++);
    memcpy(data, deb, ((end - deb) > max) ? max : end - deb);
  }
  return (0);
}

int	parsenum(char *buf, int *data)
{
  char	*deb;
  char	*end;
  char	temp[32];

  memset(temp, 0, 32);
  for (deb = buf; *deb && isspace(*deb); deb++);
  if (!*deb)
    return (-1);
  if ('"' == *deb) {
    for (end = ++deb; *end && ('"' != *end); end++);
    if (!*end)
      return (-1);
    memcpy(temp, deb, end - deb);
    *data = atoi(temp);
  }
  else {
    for (end = deb; *end && !isspace(*end); end++);
    memcpy(temp, deb, end - deb);
    *data = atoi(temp);
  }
  return (0);
}

char		**build_arg_list(char *buf, int len)
{
  int		espaces;
  int		i, j;
  char		**retour;

  /* count number of args */
  for (espaces = 0, i = 0; buf[i] && i < len; i++)
    if (isspace(buf[i]))
      espaces++;
  /* allocate space for the structure */
  if (NULL == (retour = (char **) malloc(sizeof(char *) * espaces + 2)))
    return (NULL);
  /* get each arg one by one */
  for (i = 0, j = 0; j < len && i < 256; i++) {
    /* get the end of the arg */
    for (espaces = j; espaces < len && !isspace(buf[espaces]); espaces++);
    if (0 == (retour[i] = malloc(sizeof(char) * (espaces - j) + 1))) {
      for (j = 0; j < i; j++)
	free(retour[j]);
      return (NULL);
    }
    memcpy(retour[i], buf +  j, espaces - j);
    retour[i][espaces - j] = '\0';
    j = espaces + 1;
  }
  retour[i] = 0;
  return (retour);
}


int		readConfigFile( char *filename ){
    char	buf[256];
    char	tmp[256];
    FILE	*fp;

    if( (fp = fopen( filename, "r")) == 0 ){
        sprintf(config_file, "%s/.wmpop3rc", getenv("HOME"));
        fprintf(stderr, "-Config file does not exit : %s\n",config_file);
        fprintf(stderr, "+Trying to create new config file.\n");
        if((fp = fopen(config_file,"w")) == 0){
            fprintf(stderr, "-Error creating new config file\n");
            return -1;
        }
        fprintf(fp, "autochecking       0   # 1 enables, 0 disables\n");
	fprintf(fp, "displaydelay       2   # nb of seconds error info is displayed\n");
        fprintf(fp, "viewallmessages    0   # 0 Shows the from and subject of max 19 mesgs\n");
        fprintf(fp, "#                        1 Shows the number of messages\n");
        fprintf(fp, "#\n# Replace all < > with appropriate data\n#\n");
	fprintf(fp, "[Server]               # server section\n");
	fprintf(fp, "alias              <3 alphanum. char. long alias>\n");
        fprintf(fp, "popserver          < pop3 server name >\n");
        fprintf(fp, "port               110 # default port\n");
        fprintf(fp, "username           < pop3 login name  >\n");
        fprintf(fp, "password           < pop3 password   >\n");
	fprintf(fp, "mailclient         \"netscape -mail\" # for example...\n");
        fprintf(fp, "mailcheckdelay     10  # default mail check time in minutes\n");
	fprintf(fp, "countUnreadOnly    0   # count unread messages only\n");
	fprintf(fp, "#\n# start new [server] section below (up to 6)\n");
        fprintf(stderr, "+New config file created : ~/.wmpop3rc\n\n");
        fprintf(stderr, "+ ~/.wmpop3rc must be configured before running wmpop3.\n");
        fclose(fp);
        return -1;
    }
    
    nb_conf = 0;
    
    while ((nb_conf < 7) && fgets(buf, 256, fp) != 0) {
      if (buf[0] != '#') {
	if (!nb_conf && !strncasecmp( buf, "autochecking", 12) ){
	  if (parsenum(buf + 12, &autoChecking)) 
	    fprintf(stderr, "syntax error for parameter autochecking\n");
	}
	else if (!nb_conf && !strncasecmp( buf, "displaydelay", 12) ){
	  if (parsenum(buf + 12, &displaydelay))
	    fprintf(stderr, "syntax error for parameter displaydelay\n");
	}
	else if (!strncasecmp(buf, "[server]", 8)) {
	  nb_conf++;
	  if (!(conf[nb_conf - 1] = pop3Create(nb_conf))) {
	    fprintf(stderr, "Can't allocate memory for config structure\n");
	    fclose(fp);
	    return (-1);
	  }
	}
	else if (nb_conf && !strncasecmp(buf, "username", 8) ) {
	  if (parsestring(buf + 8, conf[nb_conf -1]->username, 256))
	    fprintf(stderr, "section %d: invalid syntax for username\n", 
		    nb_conf);
	}
	else if (nb_conf && !strncasecmp( buf, "password", 8) ){
	  if (parsestring(buf + 8, conf[nb_conf - 1]->password, 256))
	    fprintf(stderr, "section %d: invalid syntax for password\n", 
		    nb_conf);
	}
	else if (nb_conf && !strncasecmp(buf, "alias", 5) ) {
	  if (parsestring(buf + 5, conf[nb_conf -1]->alias, 3))
	    fprintf(stderr, "section %d: invalid syntax for alias\n", nb_conf);
	}
	else if (nb_conf && !strncasecmp( buf, "popserver", 9) ){
	  if (parsestring(buf + 9, conf[nb_conf - 1]->popserver, 128))
	    fprintf(stderr, "section %d: invalid syntax for popserver\n", 
		    nb_conf);
	}
	else if (nb_conf && !strncasecmp( buf, "port", 4) ){
	  if (parsenum(buf + 4, &(conf[nb_conf - 1]->serverport)))
	    fprintf(stderr, "section %d: Invalid popserver port number.\n", 
		    nb_conf);
	}
	else if (!nb_conf && !strncasecmp( buf, "viewallmessages", 15) ){
	  if (parsenum(buf + 15, &newMessagesOnly))
	    fprintf(stderr, "section %d: Invalid number ( viewallmessages )\n",
		    nb_conf);
	}
	else if (nb_conf && !strncasecmp(buf, "countunreadonly", 15)) {
	  if (parsenum(buf + 15, &(conf[nb_conf - 1]->countunreadonly)))
	    fprintf(stderr, "section %d: Invalid number ( countunreadonly )\n",
		    nb_conf);
	}
	else if (nb_conf && !strncasecmp( buf, "mailcheckdelay", 14) ){
	  if (parsenum(buf + 14, &(conf[nb_conf -1]->mailCheckDelay)))
	    fprintf(stderr, "section %d: Invalid delay time.\n", nb_conf);
	}
	else if (nb_conf && !strncasecmp(buf, "mailclient", 10)) {
	  if (parsestring(buf + 10, tmp, 256))
	    fprintf(stderr, "section %d: Invalid syntax for mailclient.\n", 
		    nb_conf);
	  conf[nb_conf - 1]->mailclient = build_arg_list(tmp, strlen(tmp));
	}
	else if (nb_conf) { 
	  if (*buf && (isalpha(*buf) || isalnum(*buf)))
	    fprintf(stderr, "section %d: Unknown indentifier : [%s]\n", 
		    nb_conf, buf);
	}
	else 
	  fprintf(stderr, "identifier outside Server section\n");
      } 
    }
    fclose(fp);
    return 0;
}







